package be.jeffcheasey88.peeratcode.model;

import java.util.Arrays;

public class Completion {
	private int puzzleId;
	private int playerId;
	private int tries;
	private String fileName;
	private byte[] code;
	private int score;
	
	public Completion(int playerId, int puzzleId, int tries, String fileName, int score) {
		this(playerId, puzzleId, tries, score, fileName, null, null, null);
	}

	public Completion(int playerId, int puzzleId, String fileName, byte[] file, byte[] response, Puzzle currentPuzzle) {
		this(playerId, puzzleId, 0, 0, fileName, file, response, currentPuzzle);
	}

	public Completion(int playerId, int puzzleId, int tries, int score, String fileName, byte[] file, byte[] response,
			Puzzle currentPuzzle) {
		this.playerId = playerId;
		this.puzzleId = puzzleId;
		this.fileName = fileName;
		if (code != null)
			this.code = Arrays.copyOf(file, file.length);

		// Updated with addTry
		this.tries = tries;
		this.score = score;

		if (currentPuzzle != null)
			addTry(currentPuzzle, response);
	}

	public int getPuzzleId() {
		return puzzleId;
	}

	public int getPlayerId() {
		return playerId;
	}

	public int getTries() {
		return tries;
	}

	public void addTry(Puzzle currentPuzzle, byte[] response) {
		if (score <= 0) {
			tries++;
			if (response != null && Arrays.equals(currentPuzzle.getSoluce(), response)) {
				if (tries > 1) { // Loose 5% each try with a minimum of 1 for score
					score = (int) Math.ceil(currentPuzzle.getScoreMax() * (1 - ((tries - 1) / 20.)));
					if (score < 1)
						score = 1;
				} else
					score = currentPuzzle.getScoreMax();
			}
		}
	}

	public String getFileName() {
		return fileName;
	}

	public byte[] getCode() {
		return code;
	}

	public int getScore() {
		return score;
	}
}
